@@ -1,4 +1,5 @@
-//#ident "$Id: Pinball.cpp,v 1.44 2003/06/18 10:43:44 henqvist Exp $"
+// -*- mode: C++; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// #ident "$Id: Pinball.cpp $"
 /***************************************************************************
                           Pinball.cpp  -  description
                              -------------------
@@ -60,6 +61,8 @@
 #include <SDL.h>
 #endif
 
+using namespace std;
+
 #define AMBIENT 0.05f
 
 void get_config(void);
@@ -151,13 +154,32 @@ protected:
     } else {
       key = string("unknown");
     }
+
+	if (name.length() + key.length() > 15){  
+	    std::size_t found = name.find("leftflip");
+	if (found!=std::string::npos)
+	    name = "LF";
+	found = name.find("rightflip");
+	if (found!=std::string::npos)
+	    name = "RF";
+	found = name.find("leftnudge");
+	if (found!=std::string::npos)
+	     name = "LN";
+	found = name.find("rightnudge");
+	if (found!=std::string::npos)
+	     name = "RN";
+	found = name.find("bottomnudge");
+	if (found!=std::string::npos)
+	     name = "BN";
+	}
+
     name = name + ":";
-    while (name.size() < 12) {
+    /*while (name.size() < 12) {
       name = name + " ";
     }
     while (key.size() < 12) {
       key = " " + key;
-    }
+    }*/
     stm << name << key << '\0';
     string text = stm.str();
     return text.c_str();
@@ -247,6 +269,7 @@ protected:
     case 4: w = 800; h = 600; break;
     case 5: w = 1024; h = 768; break;
     case 6: w = 1280; h = 1024; break;
+    case 7: w = 1920; h = 1200; break;
     default: w = 640; h = 480;
     }
     if (config->getWidth() != w) {
@@ -276,6 +299,7 @@ protected:
     case 1: config->setView(1); break;
     case 2: config->setView(2); break;
     case 3: config->setView(3); break;
+    case 4: config->setView(4); break;
     default: config->setView(0);
     }
     // texture filter
@@ -441,6 +465,8 @@ void get_config(void) {
     menusize->setCurrent(5);
   } else 	if (Config::getInstance()->getWidth() == 1280) {
     menusize->setCurrent(6);
+  } else 	if (Config::getInstance()->getWidth() == 1920) {
+    menusize->setCurrent(7);
   } else {
     menusize->setCurrent(3);
   }
@@ -449,6 +475,7 @@ void get_config(void) {
   case 1: menuview->setCurrent(1); break;
   case 2: menuview->setCurrent(2); break;
   case 3: menuview->setCurrent(3); break;
+  case 4: menuview->setCurrent(4); break;
   default: menuview->setCurrent(0);
   }
   // texture filter
@@ -483,7 +510,7 @@ void get_config(void) {
 MenuItem* createMenus(Engine * engine) {
   // Create the meny
   MenuSub* menu = new MenuSub("main menu", engine);
-  menu->setBottomText("http://pinball.sourceforge.net");
+  menu->setBottomText(PACKAGE_URL " " PACKAGE_VERSION);
 
   MenuSub* menuresume = new MenuSub("play", engine);
   menuresume->setAction(EM_MENU_RESUME);
@@ -511,7 +538,8 @@ MenuItem* createMenus(Engine * engine) {
   menucfg->addMenuItem(menuaudio);
 
   MenuSub* menukey = new MenuSub("keyboard", engine);
-  menukey->setBottomText("shorcuts for view change F5-F8");
+  //menukey->setBottomText("shorcuts for view change F5-F8");
+  menukey->setBottomText("L-eft R-ight B-ottom F-lip N-udge");
   menucfg->addMenuItem(menukey);
 
   string filename = string(Config::getInstance()->getDataSubDir()) + "/splash.png";
@@ -591,6 +619,7 @@ MenuItem* createMenus(Engine * engine) {
   menuview->addText(  "view:   softly moving");
   menuview->addText(  "view:          moving");
   menuview->addText(  "view:             top");
+  menuview->addText(  "view:            full");
   menugfx->addMenuItem(menuview);
 
   menuscreen = new MenuChoose(engine);
@@ -615,6 +644,7 @@ MenuItem* createMenus(Engine * engine) {
   menusize->addText(  "screen size:  800x600");
   menusize->addText(  "screen size: 1024x768");
   menusize->addText(  "screen size:1280x1024");
+  menusize->addText(  "screen size:1920x1200");
   menugfx->addMenuItem(menusize);
 
   menufilter = new MenuChoose(engine);
@@ -699,104 +729,181 @@ MenuItem* createMenus(Engine * engine) {
   return menu;
 }
 
+Pinball::Pinball()
+  : mpMenu{}, mpEngine{}, miCount{}, miMaxCount{}
+{
+  char* var= getenv("PINBALL_QUIT");
+  if (var) {
+    if (*var) { miMaxCount = atoi(var); }
+    else { miMaxCount = 1; }
+  }
+}
 
-/** Main */
-int main(int argc, char *argv[]) {
-  //cerr<<"+ Pinball::main"<<endl;
+int Pinball::run(int argc, char *argv[])
+{
+  int status = 0;
   try {
-    // Create a engine and parse emilia arguments
-    Config::getInstance()->loadConfig();
-    Engine * engine = new Engine(argc, argv);
-
-    float direct = 0.0f;
-    if (Config::getInstance()->getBrightness() < 0.35f) {
-      direct = 0.3f;
-    } else if (Config::getInstance()->getBrightness() < 0.45f) {
-      direct = 0.4f;
-    } else if (Config::getInstance()->getBrightness() < 0.55f) {
-      direct = 0.5f;
-    } else if (Config::getInstance()->getBrightness() < 0.65f) {
-      direct = 0.6f;
-    } else if (Config::getInstance()->getBrightness() < 0.75f) {
-      direct = 0.7f;
-    } else {
-      direct = 0.8f;
+    status |= setup(argc, argv);
+    while (status == 0) {
+      status = loop();
     }
-    engine->setLightning(direct, AMBIENT);
+    switch (status) {
+    case -EM_MENU_EXIT:
+      status = EXIT_SUCCESS;
+      break;
+    default:
+      if (status!=0) { cout<<"log: run: status="<<status<<endl;}
+    }
+    if (status>0) status=0;
+  } catch(string error) {
+    status |= 0xF0;
+  }
+  status |= finish(status);
+  return status;
+}
+
+
+int Pinball::setup(int argc, char *argv[])
+{
+  // Create a engine and parse emilia arguments
+  Config::getInstance()->loadConfig();
+  mpEngine = new Engine(argc, argv);
+
+  float direct = 0.0f;
+  if (Config::getInstance()->getBrightness() < 0.35f) {
+    direct = 0.3f;
+  } else if (Config::getInstance()->getBrightness() < 0.45f) {
+    direct = 0.4f;
+  } else if (Config::getInstance()->getBrightness() < 0.55f) {
+    direct = 0.5f;
+  } else if (Config::getInstance()->getBrightness() < 0.65f) {
+    direct = 0.6f;
+  } else if (Config::getInstance()->getBrightness() < 0.75f) {
+    direct = 0.7f;
+  } else {
+    direct = 0.8f;
+  }
+  mpEngine->setLightning(direct, AMBIENT);
 
 #if EM_USE_SDL
-    string filename = Config::getInstance()->getDataDir() + string("/font_34.png");
+  string filename = Config::getInstance()->getDataDir() + string("/font_34.png");
 #endif
 #if EM_USE_ALLEGRO
-    string filename = Config::getInstance()->getDataDir() + string("/font_35.pcx");
+  string filename = Config::getInstance()->getDataDir() + string("/font_35.pcx");
 #endif
 
-    EmFont::getInstance()->loadFont(filename.c_str());
+  EmFont::getInstance()->loadFont(filename.c_str());
 
-    // Add a score board and a menu.
-    MenuItem* menu = createMenus(engine);
+  // Add a score board and a menu.
+  mpMenu = createMenus(mpEngine);
 
-    // Draw to the screen.
-    int all = 0;
+  // cerr<<"log: pinball: Draw to the screen"<<endl;
+  miCount = 0;
 
-    engine->resetTick();
+  mpEngine->resetTick();
+  mpEngine->swap();
 
-    while (! (  SDL_QuitRequested()  //cout<<"catch close win"<<endl; //!rzr
-                || Keyboard::isKeyDown(SDLK_INSERT)))  {
-#if EM_DEBUG
-      if (Keyboard::isKeyDown(SDLK_p)) {
-        Keyboard::waitForKey();
-        Keyboard::clear();
-        engine->resetTick();
-      }
+  char const * table = 0;
+  if (miCount == 0 && !mpEngine->getGroup(0)) {
+    if (table = getenv("PINBALL_TABLE")) {
+      if (table && !*table) {
+	int iTable = 0;
+#if ! EM_DEBUG
+	srand(time(NULL));
+	iTable = rand() % 100 +1; // TODO reload last table from config
 #endif
-      if (Keyboard::isKeyDown(SDLK_ESCAPE) || all == 0) {
-        SoundUtil::getInstance()->pauseMusic();
-        if (menu->perform() == EM_MENU_EXIT) {
-          break;
-        }
-        engine->resetTick();
-        SoundUtil::getInstance()->resumeMusic();
+	table = (iTable<51) ? "professor" : "tux";
       }
-
-      if (Keyboard::isKeyDown(SDLK_r)) {
-        SendSignal(PBL_SIG_RESET_ALL, 0, engine, NULL);
-      }
-
-      if (engine->nextTickFPS(200)) {
-        engine->tick();
-      } else {
-        engine->render();
-        if (Table::getInstance()->getScore() != NULL) {
-          Table::getInstance()->getScore()->draw();
-        }
-        if (engine->getGroup(0) == NULL) {
-          EmFont::getInstance()->printRowCenter("no table loaded", 6);
-          EmFont::getInstance()->printRowCenter("press esc", 8);
-        }
-        engine->swap();
-      }
-
-      all++;
-      //engine->limitFPS(100);
+      cout<<"log: pinball: Load table on init"<<endl;
+      if (Table::getInstance()->loadLevel(mpEngine, table) == 0)
+	Table::getInstance()->readHighScoresFile();
+      Keyboard::generate(SDLK_RETURN);
+      Keyboard::clear();
     }
-
-    Config::getInstance()->saveConfig();
-
-    // Write high scores to disk - pnf
-    Table::getInstance()->writeHighScoresFile();
-
-    delete(engine);
-  } catch (string str) {
-    cerr << "EXCEPTION: " << str << endl;
   }
+  return 0;
+}
+
+
+int Pinball::loop()
+{
+  if (miMaxCount && miCount >= miMaxCount) {
+    return 1;
+  }
+#if EM_USE_SDL
+  if (SDL_QuitRequested())
+    return 2;
+#if EM_DEBUG
+  if (Keyboard::isKeyDown(SDLK_INSERT))
+    return 3;
+#endif
+#endif
+#if EM_DEBUG
+  if (Keyboard::isKeyDown(SDLK_p)) {
+    Keyboard::waitForKey();
+    Keyboard::clear();
+    mpEngine->resetTick();
+  }
+#endif
+  if (mpEngine->getGroup(0)) {
+    if (miCount++ == 0) {
+      cerr<<"log: Start table if loaded: "<< miCount<<endl;
+      return 0;
+    }
+  }
+  
+  if (Keyboard::isKeyDown(SDLK_ESCAPE) || miCount == 0) {
+    SoundUtil::getInstance()->pauseMusic();
+    if (mpMenu->perform() == EM_MENU_EXIT) {
+      return -EM_MENU_EXIT;
+    }
+    mpEngine->resetTick();
+    SoundUtil::getInstance()->resumeMusic();
+  }
+
+  if (Keyboard::isKeyDown(SDLK_r)) {
+    SendSignal(PBL_SIG_RESET_ALL, 0, mpEngine, NULL);
+  }
+  if (mpEngine->nextTickFPS(200)) {
+    mpEngine->tick();
+  } else {
+    mpEngine->render();
+    if (Table::getInstance()->getScore() != NULL) {
+      Table::getInstance()->getScore()->draw();
+    } else {
+      EmFont::getInstance()->printRowCenter("no table loaded", 6);
+      EmFont::getInstance()->printRowCenter("press esc", 8);
+    }
+    mpEngine->swap();
+  }
+  miCount++;
+#if EM_DEBUG
+  //engine->limitFPS(100); // TODO: remove if no relevent
+#endif
   return EXIT_SUCCESS;
 }
 
-#if EM_USE_ALLEGRO
-END_OF_MAIN();
-#endif
+int Pinball::finish(int status)
+{
+  Config::getInstance()->saveConfig();
 
+    // Write high scores to disk - pnf
+  Table::getInstance()->writeHighScoresFile();
+  delete(mpEngine); mpEngine=0;
+  return EXIT_SUCCESS;
+}
+
+/// Main
+
+int Pinball::main(int argc, char *argv[])
+{
+  Pinball instance;
+  int status = instance.run(argc, argv);
+  cout<<"pinball: exit: "<<status<<endl;
+  return status;
+}
+
+int main(int argc=0, char *argv[]=0);
 
 /// entry point function (main) for w32 codewarrior
 #if( (defined WIN32 ) && ( defined __MWERKS__ ) )
@@ -874,4 +981,17 @@ int WINAPI WinMain( HINSTANCE hInst,  HINSTANCE hPreInst,
   return main(argc,argv);
 }
 #endif
-// EOF $Id: Pinball.cpp,v 1.44 2003/06/18 10:43:44 henqvist Exp $
+
+
+int main(int argc, char *argv[])
+{
+  int status = EXIT_SUCCESS;
+  status |=  Pinball::main(argc, argv);
+  return status;
+}
+
+#if EM_USE_ALLEGRO
+END_OF_MAIN();
+#endif
+
+// #eof "$Id: Pinball.cpp $"
